@@ -1,4 +1,7 @@
 import argparse
+
+import six
+
 from sphinxarg.parser import parse_parser, parser_navigate
 
 
@@ -10,15 +13,8 @@ def test_parse_options():
     data = parse_parser(parser)
 
     assert data['action_groups'][0]['options'] == [
-        {
-            'name': ['--foo'],
-            'default': False,
-            'help': 'foo help'
-        }, {
-            'name': ['--bar'],
-            'default': False,
-            'help': ''
-        }
+        {'name': ['--foo'], 'default': False, 'help': 'foo help'},
+        {'name': ['--bar'], 'default': False, 'help': ''},
     ]
 
 
@@ -29,11 +25,7 @@ def test_parse_default():
     data = parse_parser(parser)
 
     assert data['action_groups'][0]['options'] == [
-        {
-            'name': ['--foo'],
-            'default': '"123"',
-            'help': ''
-        }
+        {'name': ['--foo'], 'default': '"123"', 'help': ''}
     ]
 
 
@@ -48,7 +40,7 @@ def test_parse_arg_choices():
             'name': ['move'],
             'help': '',
             'choices': ['rock', 'paper', 'scissors'],
-            'default': None
+            'default': None,
         }
     ]
 
@@ -64,7 +56,7 @@ def test_parse_opt_choices():
             'name': ['--move'],
             'default': None,
             'help': '',
-            'choices': ['rock', 'paper', 'scissors']
+            'choices': ['rock', 'paper', 'scissors'],
         }
     ]
 
@@ -76,11 +68,7 @@ def test_parse_default_skip_default():
     data = parse_parser(parser, skip_default_values=True)
 
     assert data['action_groups'][0]['options'] == [
-        {
-            'name': ['--foo'],
-            'default': '==SUPPRESS==',
-            'help': ''
-        }
+        {'name': ['--foo'], 'default': '==SUPPRESS==', 'help': ''}
     ]
 
 
@@ -92,15 +80,8 @@ def test_parse_positional():
     data = parse_parser(parser)
 
     assert data['action_groups'][0]['options'] == [
-        {
-            'name': ['foo'],
-            'help': 'foo help',
-            'default': False
-        }, {
-            'name': ['bar'],
-            'help': '',
-            'default': False
-        }
+        {'name': ['foo'], 'help': 'foo help', 'default': False},
+        {'name': ['bar'], 'help': '', 'default': False},
     ]
 
 
@@ -116,20 +97,13 @@ def test_parse_description():
     assert data['epilog'] == 'epilogged'
 
     assert data['action_groups'][0]['options'] == [
-        {
-            'name': ['foo'],
-            'help': 'foo help',
-            'default': False
-        }, {
-            'name': ['bar'],
-            'help': '',
-            'default': False
-        }
+        {'name': ['foo'], 'help': 'foo help', 'default': False},
+        {'name': ['bar'], 'help': '', 'default': False},
     ]
 
 
 def test_parse_nested():
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(prog='under-test')
     parser.add_argument('foo', default=False, help='foo help')
     parser.add_argument('bar', default=False)
 
@@ -137,58 +111,116 @@ def test_parse_nested():
 
     subparser = subparsers.add_parser('install', help='install help')
     subparser.add_argument('ref', type=str, help='foo1 help')
-    subparser.add_argument('--upgrade', action='store_true', default=False, help='foo2 help')
+    subparser.add_argument(
+        '--upgrade', action='store_true', default=False, help='foo2 help'
+    )
 
     data = parse_parser(parser)
 
     assert data['action_groups'][0]['options'] == [
-        {
-            'name': ['foo'],
-            'help': 'foo help',
-            'default': False
-        }, {
-            'name': ['bar'],
-            'help': '',
-            'default': False
-        }
+        {'name': ['foo'], 'help': 'foo help', 'default': False},
+        {'name': ['bar'], 'help': '', 'default': False},
     ]
 
     assert data['children'] == [
         {
             'name': 'install',
             'help': 'install help',
-            'usage': 'usage: py.test install [-h] [--upgrade] ref',
-            'bare_usage': 'py.test install [-h] [--upgrade] ref',
+            'usage': 'usage: under-test install [-h] [--upgrade] ref',
+            'bare_usage': 'under-test install [-h] [--upgrade] ref',
             'action_groups': [
                 {
                     'title': 'Positional Arguments',
                     'description': None,
                     'options': [
-                        {
-                            'name': ['ref'],
-                            'help': 'foo1 help',
-                            'default': None
-                        }
-                    ]
+                        {'name': ['ref'], 'help': 'foo1 help', 'default': None}
+                    ],
                 },
                 {
                     'description': None,
                     'title': 'Named Arguments',
                     'options': [
-                        {
-                            'name': ['--upgrade'],
-                            'default': False,
-                            'help': 'foo2 help'
-                        }
-                    ]
-                }
-            ]
+                        {'name': ['--upgrade'], 'default': False, 'help': 'foo2 help'}
+                    ],
+                },
+            ],
         }
     ]
 
 
+if six.PY3:
+
+    def test_parse_nested_with_alias():
+        parser = argparse.ArgumentParser(prog='under-test')
+        parser.add_argument('foo', default=False, help='foo help')
+        parser.add_argument('bar', default=False)
+
+        subparsers = parser.add_subparsers()
+
+        subparser = subparsers.add_parser('install', aliases=['i'], help='install help')
+        subparser.add_argument('ref', type=str, help='foo1 help')
+        subparser.add_argument(
+            '--upgrade', action='store_true', default=False, help='foo2 help'
+        )
+
+        data = parse_parser(parser)
+
+        assert data['action_groups'][0]['options'] == [
+            {'name': ['foo'], 'help': 'foo help', 'default': False},
+            {'name': ['bar'], 'help': '', 'default': False},
+        ]
+
+        assert data['children'] == [
+            {
+                'name': 'install (i)',
+                'identifier': 'install',
+                'help': 'install help',
+                'usage': 'usage: under-test install [-h] [--upgrade] ref',
+                'bare_usage': 'under-test install [-h] [--upgrade] ref',
+                'action_groups': [
+                    {
+                        'title': 'Positional Arguments',
+                        'description': None,
+                        'options': [
+                            {'name': ['ref'], 'help': 'foo1 help', 'default': None}
+                        ],
+                    },
+                    {
+                        'description': None,
+                        'title': 'Named Arguments',
+                        'options': [
+                            {
+                                'name': ['--upgrade'],
+                                'default': False,
+                                'help': 'foo2 help',
+                            }
+                        ],
+                    },
+                ],
+            }
+        ]
+
+    def test_aliased_traversal():
+        parser = argparse.ArgumentParser(prog='under-test')
+
+        subparsers1 = parser.add_subparsers()
+        subparsers1.add_parser('level1', aliases=['l1'])
+
+        data = parse_parser(parser)
+
+        data2 = parser_navigate(data, 'level1')
+
+        assert data2 == {
+            'bare_usage': 'under-test level1 [-h]',
+            'help': '',
+            'usage': 'usage: under-test level1 [-h]',
+            'name': 'level1 (l1)',
+            'identifier': 'level1',
+        }
+
+
 def test_parse_nested_traversal():
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(prog='under-test')
 
     subparsers1 = parser.add_subparsers()
     subparser1 = subparsers1.add_parser('level1')
@@ -207,15 +239,8 @@ def test_parse_nested_traversal():
     data3 = parser_navigate(data, 'level1 level2 level3')
 
     assert data3['action_groups'][0]['options'] == [
-        {
-            'name': ['foo'],
-            'help': 'foo help',
-            'default': None
-        }, {
-            'name': ['bar'],
-            'help': '',
-            'default': None
-        }
+        {'name': ['foo'], 'help': 'foo help', 'default': None},
+        {'name': ['bar'], 'help': '', 'default': None},
     ]
 
     data2 = parser_navigate(data, 'level1 level2')
@@ -223,25 +248,18 @@ def test_parse_nested_traversal():
         {
             'name': 'level3',
             'help': '',
-            'usage': 'usage: py.test level1 level2 level3 [-h] foo bar',
-            'bare_usage': 'py.test level1 level2 level3 [-h] foo bar',
+            'usage': 'usage: under-test level1 level2 level3 [-h] foo bar',
+            'bare_usage': 'under-test level1 level2 level3 [-h] foo bar',
             'action_groups': [
                 {
                     'title': 'Positional Arguments',
                     'description': None,
                     'options': [
-                        {
-                            'default': None,
-                            'name': ['foo'],
-                            'help': 'foo help'
-                        }, {
-                            'name': ['bar'],
-                            'help': '',
-                            'default': None
-                        }
-                    ]
+                        {'default': None, 'name': ['foo'], 'help': 'foo help'},
+                        {'name': ['bar'], 'help': '', 'default': None},
+                    ],
                 }
-            ]
+            ],
         }
     ]
 
@@ -260,7 +278,7 @@ def test_fill_in_default_prog():
         {
             'default': '"foo"',
             'name': ['bar'],
-            'help': 'test_fill_in_default_prog (default: "foo")'
+            'help': 'test_fill_in_default_prog (default: "foo")',
         }
     ]
 
@@ -271,14 +289,16 @@ def test_string_quoting():
     This prevents things like '--optLSFConf=-q short' when '--optLSFConf="-q short"' is correct.
     """
     parser = argparse.ArgumentParser(prog='test_string_quoting_prog')
-    parser.add_argument('--bar', default='foo bar', help='%(prog)s (default: %(default)s)')
+    parser.add_argument(
+        '--bar', default='foo bar', help='%(prog)s (default: %(default)s)'
+    )
     data = parse_parser(parser)
 
     assert data['action_groups'][0]['options'] == [
         {
             'default': '"foo bar"',
             'name': ['--bar'],
-            'help': 'test_string_quoting_prog (default: "foo bar")'
+            'help': 'test_string_quoting_prog (default: "foo bar")',
         }
     ]
 
@@ -297,15 +317,18 @@ def test_parse_groups():
             'description': None,
             'options': [
                 {'default': False, 'help': 'foo help', 'name': ['--foo']},
-                {'default': False, 'help': '', 'name': ['--bar']}],
-            'title': 'Named Arguments'},
+                {'default': False, 'help': '', 'name': ['--bar']},
+            ],
+            'title': 'Named Arguments',
+        },
         {
             'description': None,
             'options': [
                 {'default': None, 'help': 'option #1', 'name': ['--option1']},
-                {'default': None, 'help': 'option #2', 'name': ['--option2']}],
-            'title': 'Group 1'
-        }
+                {'default': None, 'help': 'option #2', 'name': ['--option2']},
+            ],
+            'title': 'Group 1',
+        },
     ]
 
 
@@ -334,13 +357,70 @@ def test_action_groups_with_subcommands():
     data = parse_parser(parser)
 
     assert data['action_groups'] == [
-        {'options': [{'default': None, 'name': ['foo2 metavar'], 'help': 'foo2 help'}], 'description': None, 'title': 'Positional Arguments'},
-        {'options': [{'default': None, 'name': ['--foo'], 'help': 'foo help'}], 'description': None, 'title': 'Named Arguments'},
-        {'options': [{'default': None, 'name': ['--bar'], 'help': 'bar help'}, {'default': None, 'name': ['quux'], 'help': 'quux help'}], 'description': None, 'title': 'bar options'},
-        {'options': [{'default': None, 'name': ['--blah'], 'help': 'blah help'}, {'default': None, 'name': ['sniggly'], 'help': 'sniggly help'}], 'description': None, 'title': 'bla options'}
+        {
+            'options': [
+                {'default': None, 'name': ['foo2 metavar'], 'help': 'foo2 help'}
+            ],
+            'description': None,
+            'title': 'Positional Arguments',
+        },
+        {
+            'options': [{'default': None, 'name': ['--foo'], 'help': 'foo help'}],
+            'description': None,
+            'title': 'Named Arguments',
+        },
+        {
+            'options': [
+                {'default': None, 'name': ['--bar'], 'help': 'bar help'},
+                {'default': None, 'name': ['quux'], 'help': 'quux help'},
+            ],
+            'description': None,
+            'title': 'bar options',
+        },
+        {
+            'options': [
+                {'default': None, 'name': ['--blah'], 'help': 'blah help'},
+                {'default': None, 'name': ['sniggly'], 'help': 'sniggly help'},
+            ],
+            'description': None,
+            'title': 'bla options',
+        },
     ]
 
     assert data['children'] == [
-        {'usage': 'usage: foo A [-h] baz', 'action_groups': [{'options': [{'default': None, 'name': ['baz'], 'help': 'An integer'}], 'description': None, 'title': 'Positional Arguments'}], 'bare_usage': 'foo A [-h] baz', 'name': 'A', 'help': 'A subparser'},
-        {'usage': 'usage: foo B [-h] [--barg {X,Y,Z}]', 'action_groups': [{'options': [{'default': None, 'choices': 'XYZ', 'name': ['--barg'], 'help': 'A list of choices'}], 'description': None, 'title': 'Named Arguments'}], 'bare_usage': 'foo B [-h] [--barg {X,Y,Z}]', 'name': 'B', 'help': 'B subparser'}
+        {
+            'usage': 'usage: foo A [-h] baz',
+            'action_groups': [
+                {
+                    'options': [
+                        {'default': None, 'name': ['baz'], 'help': 'An integer'}
+                    ],
+                    'description': None,
+                    'title': 'Positional Arguments',
+                }
+            ],
+            'bare_usage': 'foo A [-h] baz',
+            'name': 'A',
+            'help': 'A subparser',
+        },
+        {
+            'usage': 'usage: foo B [-h] [--barg {X,Y,Z}]',
+            'action_groups': [
+                {
+                    'options': [
+                        {
+                            'default': None,
+                            'choices': 'XYZ',
+                            'name': ['--barg'],
+                            'help': 'A list of choices',
+                        }
+                    ],
+                    'description': None,
+                    'title': 'Named Arguments',
+                }
+            ],
+            'bare_usage': 'foo B [-h] [--barg {X,Y,Z}]',
+            'name': 'B',
+            'help': 'B subparser',
+        },
     ]
